#version 440 core

// ==================================================
// Código 'Vertex Shader'
// Este código (paralelizado) corre en la GPU para cada uno de los vértices de un polígono del escenario 3D, ya sea un objeto 3D o un plano.
// Este código está escrito en lenguaje 'GLSL' (OpenGL Shading Language) y es muy similar al lenguaje 'C'.
// El código lo compila una API del propio OpenGL antes de pasarlo a la GPU (no lo compila MS Visual C++)
// ==================================================

// Variables de entrada. Son variables del sistema.
layout (location = 0) in vec3 vertex_position;
layout (location = 1) in vec3 vertex_color;
layout (location = 2) in vec2 vertex_uv;
layout (location = 3) in vec3 vertex_normals;

// Variables de salida hacia el código de 'Fragment Shader'
out vec4 vertexColor;
out vec2 vertexUv;
out vec3 vertexNorms;
out vec3 fragPos;
out float visibility;

// Constantes para la niebla
const float fogDensity = 0.0010f;
const float gradient = 7.0f;

// Variables de entrada. Se las pasa el código del programa de la CPU en cada 'frame'.
uniform mat4 model;
uniform mat4 view;
uniform mat4 projection;

varying float distToCamera;

// =============================================================================
// Función inicial
// =============================================================================
void main()
{
	// 'gl_Position' es una variable del sistema que indicará la posición de este vértice.
    gl_Position = projection * view *  model * vec4(vertex_position, 1.0f);
	vertexColor = vec4(vertex_color, 1.0f);
	vertexUv = vertex_uv;
	fragPos = vec3(model * vec4(vertex_position, 1.0f));
	vertexNorms = mat3(transpose(inverse(model))) * vertex_normals;

	// Cálculo de la variable 'visibility' que es la distancia entre este vértice y la posición de la cámara del jugador.
	// 'visibility' la usa el código de 'Fragment Shader' para saber cuanto de visible es cada pixel según su lejanía.
	distToCamera = gl_Position.w;	// Distancia desde la cámara hasta el vértice
	if (distToCamera < 0.0f) distToCamera = 1.0f;
	visibility = exp(-pow((distToCamera * fogDensity), gradient));
		
	// --- Fin del proceso 'Vertex Shader' ---
}