#version 440 core

layout (location = 0) in vec3 vertex_position;
layout (location = 1) in vec3 vertex_color;
layout (location = 2) in vec2 vertex_uv;
layout (location = 3) in vec3 vertex_normals;
layout (location = 4) in vec3 vertex_tangent;

out vec4 vertexColor; 
out vec2 vertexUv;
out vec3 vertexNorms;
out vec3 fragPos;
out mat3 TBN;

// Fog items 
out float visibility; 
const float fogDensity = 0.0010f;
const float gradient = 7.0f;

uniform mat4 model;
uniform mat4 view;
uniform mat4 projection;

varying float distToCamera;

void main()
{
    gl_Position = projection * view *  model * vec4(vertex_position, 1.0f);
	vertexColor = vec4(vertex_color, 1.0f);
	vertexUv = vertex_uv;
	fragPos = vec3(model * vec4(vertex_position, 1.0f));
	//vertexNorms = mat3(transpose(inverse(model))) * vertex_normals;

	vec3 T = normalize(vec3(model * vec4(vertex_tangent, 0.0f)));
	vec3 N = normalize(vec3(model * vec4(vertex_normals, 0.0f)));
	vec3 B = cross(N, T);
	TBN = mat3(T, B, N);
	
	// Cálculo de la variable 'visibility' que es la distancia entre este vértice y la posición de la cámara del jugador.
	// 'visibility' la usa el código de 'Fragment Shader' para saber cuanto de visible es cada pixel según su lejanía.
	distToCamera = gl_Position.w;	// Distancia desde la cámara hasta el vértice
	visibility = exp(-pow((distToCamera * fogDensity), gradient));
}